#!/bin/bash

#Global Variables
MIRNAS_DATA_DIR="miRNAs8.2h"
NCRNAS_DATA_DIR="Rfam7.0"

function training {
	# Variables
	POS_SET="Homo_sapiens"
	NEG_SET="pseudoMiRNAs"
	POS_SET_DIR="${MIRNAS_DATA_DIR}"
	NEG_SET_DIR="pseudoMiRNAs"
	POS_NUM=200
	NEG_NUM=$(($POS_NUM*2))
	SAMPLE=${POS_NUM}${POS_SET}_${NEG_NUM}${NEG_SET}
	RAND_SEED=$1
	TIMESTAMP=$2
	SVM_MODEL=$3
	FILEEXT=$4
	
	awk -v type=${SVM_MODEL} -f SVM-models.awk ${POS_SET_DIR}/${POS_SET}.${FILEEXT} | column -t > ${POS_SET}.${FILEEXT}
	awk -v type=${SVM_MODEL} -f SVM-models.awk ${NEG_SET_DIR}/${NEG_SET}.${FILEEXT} | column -t > ${NEG_SET}.${FILEEXT}

	# Convert stats to libsvm format
	perl libsvm_format.pl 1 ${POS_SET}.${FILEEXT} ${POS_SET}.svm
	perl libsvm_format.pl -1 ${NEG_SET}.${FILEEXT} ${NEG_SET}.svm
	
	# Select randomly
	python subset.py -s 1 -r ${RAND_SEED} ${POS_SET}.svm ${POS_NUM} ${POS_SET}.svm1 ${POS_SET}.svm2
	python subset.py -s 1 -r ${RAND_SEED} ${NEG_SET}.svm ${NEG_NUM} ${NEG_SET}.svm1 ${NEG_SET}.svm2
	python subset.py -s 1 -r ${RAND_SEED} ${NEG_SET}.svm2 $(expr `cat ${POS_SET}.svm2 | wc -l | sed 's/ //g'` \* 2) ${NEG_SET}.svm3 ${NEG_SET}.svm
	cat ${POS_SET}.svm1 ${NEG_SET}.svm1 > ${SAMPLE}.train.svm
	cat ${POS_SET}.svm2 ${NEG_SET}.svm3 > ${SAMPLE}.test.svm
	rm -f ${POS_SET}.svm ${POS_SET}.svm1 ${POS_SET}.svm2 ${NEG_SET}.svm1 ${NEG_SET}.svm2 ${NEG_SET}.svm3

	# Scale the training and testing sets
	svm-scale -s ${SAMPLE}.range ${SAMPLE}.train.svm > ${SAMPLE}.train.scale
	svm-scale -r ${SAMPLE}.range ${SAMPLE}.test.svm > ${SAMPLE}.test.scale
	rm -f ${SAMPLE}.test.svm ${SAMPLE}.train.svm

	# Create F-scores
	perl fscores.pl ${POS_SET}.${FILEEXT} < ${SAMPLE}.train.scale > ${SAMPLE}.train.scale.fscores
	sort -nr -k`awk 'END {print NF}' ${SAMPLE}.train.scale.fscores` ${SAMPLE}.train.scale.fscores > ${SAMPLE}.train.scale.fscores.temp
	mv -f ${SAMPLE}.train.scale.fscores.temp ${SAMPLE}.train.scale.fscores
	rm -f ${POS_SET}.${FILEEXT} ${NEG_SET}.${FILEEXT}
	
	# Select C and gamma
	python grid.py -log2c -10,15,1 -log2g -15,10,1 -v 5 -m 400 ${SAMPLE}.train.scale | tail -1 > ${SAMPLE}.grid.best
	mv -f ${SAMPLE}.train.scale.out ${SAMPLE}.grid
	rm -f ${SAMPLE}.train.scale.png 

	# Training
	SVM_TRAIN=`awk '{printf "svm-train -b 1 -c %s -g %s", $1, $2}' ${SAMPLE}.grid.best`
	${SVM_TRAIN} ${SAMPLE}.train.scale ${SAMPLE}.model

	# Training Predictions
	svm-predict -b 1 ${SAMPLE}.train.scale ${SAMPLE}.model ${SAMPLE}.train.predict > ${SAMPLE}.train.predict.stdout
	sed '/labels/d' ${SAMPLE}.train.predict > ${SAMPLE}.train.predict.temp
	cut -d" " -f1 ${SAMPLE}.train.scale | paste ${SAMPLE}.train.predict.temp - > ${SAMPLE}.train.predict
	perl ROC.pl 3 4 -1 < ${SAMPLE}.train.predict > ${SAMPLE}.train.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.train.roc`
	awk -v auc=${AUC} -v type="${SAMPLE}.train" -v header=1 -f SVM-metrics.awk ${SAMPLE}.train.predict > ${SAMPLE}.metrics
	rm -f ${SAMPLE}.train.predict.temp

	# Testing Predictions
	svm-predict -b 1 ${SAMPLE}.test.scale ${SAMPLE}.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
	sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
	cut -d" " -f1 ${SAMPLE}.test.scale | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
	perl ROC.pl 3 4 -1 < ${SAMPLE}.test.predict > ${SAMPLE}.test.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.test.roc`
	awk -v auc=${AUC} -v type="${SAMPLE}.test" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${SAMPLE}.metrics
	rm -f ${SAMPLE}.test.predict.temp

	mkdir ${TIMESTAMP}-${RAND_SEED}-${SVM_MODEL}
	mv -f ${SAMPLE}.* ${NEG_SET}.svm ${TIMESTAMP}-${RAND_SEED}-${SVM_MODEL}
}

function test_miRNAs {

	POS_SET_DIR="${MIRNAS_DATA_DIR}"
	NEG_SET="pseudoMiRNAs"
	FIRSTTIME=1
	MODEL_DIR=$1
	RAND_SEED=$2
	SVM_MODEL=$3
	FILEEXT=$4
	
	mkdir -p ${MODEL_DIR}_${POS_SET_DIR}
	
	for i in `ls ${POS_SET_DIR}/*.${FILEEXT}`
	do
		POS_SET=`basename "$i" .${FILEEXT}`

		awk -v type=${SVM_MODEL} -f SVM-models.awk ${POS_SET_DIR}/${POS_SET}.${FILEEXT} | column -t > ${POS_SET}.${FILEEXT}
		perl libsvm_format.pl 1 ${POS_SET}.${FILEEXT} ${POS_SET}.svm
		rm -f ${POS_SET}.${FILEEXT}
		
		POS_NUM=`cat ${POS_SET}.svm | wc -l | sed 's/ //g'`
		NEG_NUM=$(($POS_NUM*2))
		SAMPLE=${POS_SET}${POS_NUM}_${NEG_SET}${NEG_NUM}
		
		if [ ! -e ${MODEL_DIR}/${NEG_SET}.svm.org ]
		then
			cp -f ${MODEL_DIR}/${NEG_SET}.svm ${MODEL_DIR}/${NEG_SET}.svm.org
			FIRSTTIME=0
		fi
		
		if [ "${FIRSTTIME}" -eq 1 ]
		then
			cp -f ${MODEL_DIR}/${NEG_SET}.svm.org ${MODEL_DIR}/${NEG_SET}.svm
			FIRSTTIME=0
		fi
		
		python subset.py -s 1 -r ${RAND_SEED} ${MODEL_DIR}/${NEG_SET}.svm ${NEG_NUM} ${NEG_SET}.svm1 ${NEG_SET}.svm2
		cat ${POS_SET}.svm ${NEG_SET}.svm1 > ${SAMPLE}.test.svm
		rm -f ${POS_SET}.svm ${NEG_SET}.svm1
		mv -f ${NEG_SET}.svm2 ${MODEL_DIR}/${NEG_SET}.svm
		
		svm-scale -r ${MODEL_DIR}/*.range ${SAMPLE}.test.svm > ${SAMPLE}.test.scale
		rm -f ${SAMPLE}.test.svm

		svm-predict -b 1 ${SAMPLE}.test.scale ${MODEL_DIR}/*.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
		sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
		cut -d" " -f1 ${SAMPLE}.test.scale | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
		rm -f ${SAMPLE}.test.predict.temp
	
		if [ ! -e ${MODEL_DIR}_${POS_SET_DIR}.metrics ]
		then
			awk -v type="${SAMPLE}" -v header=1 -f SVM-metrics.awk ${SAMPLE}.test.predict > ${MODEL_DIR}_${POS_SET_DIR}.metrics
		else
			awk -v type="${SAMPLE}" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${MODEL_DIR}_${POS_SET_DIR}.metrics
		fi
		
		mv -f ${SAMPLE}.* ${MODEL_DIR}_${POS_SET_DIR}
		
	done
}

function test_RNAs {

	DATASET_DIR=$1
	CLASS=$2
	MODEL_DIR=$3
	SVM_MODEL=$4
	FILEEXT=$5
	
	mkdir -p ${MODEL_DIR}_${DATASET_DIR}
	
	for i in `ls ${DATASET_DIR}/*.${FILEEXT}`
	do
		DATASET=`basename "$i" .${FILEEXT}`
		
		awk -v type=${SVM_MODEL} -f SVM-models.awk ${DATASET_DIR}/${DATASET}.${FILEEXT} | column -t > ${DATASET}.${FILEEXT}
		perl libsvm_format.pl ${CLASS} ${DATASET}.${FILEEXT} ${DATASET}.svm
		rm -f ${DATASET}.${FILEEXT}
		
		NUM=`cat ${DATASET}.svm | wc -l | sed 's/ //g'`
		SAMPLE=${DATASET}_${NUM}
		mv -f ${DATASET}.svm ${SAMPLE}.test.svm
		
		svm-scale -r ${MODEL_DIR}/*.range ${SAMPLE}.test.svm > ${SAMPLE}.test.scale
		rm -f ${SAMPLE}.test.svm

		svm-predict -b 1 ${SAMPLE}.test.scale ${MODEL_DIR}/*.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
		sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
		cut -d" " -f1 ${SAMPLE}.test.scale | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
		rm -f ${SAMPLE}.test.predict.temp
		
		if [ ! -e ${MODEL_DIR}_${DATASET_DIR}.metrics ]
		then
			awk -v type="${SAMPLE}" -v header=1 -f SVM-metrics.awk ${SAMPLE}.test.predict > ${MODEL_DIR}_${DATASET_DIR}.metrics
		else
			awk -v type="${SAMPLE}" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${MODEL_DIR}_${DATASET_DIR}.metrics
		fi
		
		mv -f ${SAMPLE}.* ${MODEL_DIR}_${DATASET_DIR}
		
	done
}

for SVM_MODEL in dastatsA
do 
	TIMESTAMP=`date +%H%M%S-%d%m%Y`
	NUM=125
	FILEEXT=${SVM_MODEL}
	
	if [ "`echo ${SVM_MODEL} | grep -c ^stats`" -eq 1 ]
	then	
		FILEEXT="stats"
	fi	
	
	if [ "`echo ${SVM_MODEL} | grep -c ^astats`" -eq 1 ]
	then	
		FILEEXT="astats"
	fi	
	
	if [ "`echo ${SVM_MODEL} | grep -c ^mastats`" -eq 1 ]
	then	
		FILEEXT="mastats"
	fi	
	
	if [ "`echo ${SVM_MODEL} | grep -c ^dastats`" -eq 1 ]
	then	
		FILEEXT="dastats"
	fi
	
	if [ "`echo ${SVM_MODEL} | grep -c ^zastats`" -eq 1 ]
	then	
		FILEEXT="zastats"
	fi
	
	if [ "`echo ${SVM_MODEL} | grep -c ^fastats`" -eq 1 ]
	then	
		FILEEXT="fastats"
	fi	
	
	MODEL_DIR="${TIMESTAMP}-${NUM}-${SVM_MODEL}"
	
	training ${NUM} ${TIMESTAMP} ${SVM_MODEL} ${FILEEXT}
	test_miRNAs ${MODEL_DIR} ${NUM} ${SVM_MODEL} ${FILEEXT}
	test_RNAs ${NCRNAS_DATA_DIR} "-1" ${MODEL_DIR} ${SVM_MODEL} ${FILEEXT}
	test_RNAs mRNAs "-1" ${MODEL_DIR} ${SVM_MODEL} ${FILEEXT}
	
	./Bayes-workflow.bash ${MODEL_DIR} ${FILEEXT}
	
done
